/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or (at
 *  your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  Copyright © 2006 Pierre Habouzit
 */

/*
 * Copyright notice from original mutt:
 * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

#ifndef MUTT_LIB_LIB_BUFFER_H
#define MUTT_LIB_LIB_BUFFER_H

typedef struct buffer_t {
    char *data;
    ssize_t len;
    ssize_t size;
} buffer_t;

DO_INIT(buffer_t, buffer);
static inline void buffer_wipe(buffer_t *buf) {
    p_delete(&buf->data);
}
DO_NEW(buffer_t, buffer);
DO_DELETE(buffer_t, buffer);

static inline char *buffer_unwrap(buffer_t **buf) {
    char *res = (*buf)->data;
    (*buf)->data = NULL;
    buffer_delete(buf);
    return res;
}


static inline void buffer_resize(buffer_t *buf, ssize_t newsize) {
    p_allocgrow(&buf->data, newsize + 1, &buf->size);
}
static inline void buffer_ensure(buffer_t *buf, ssize_t extra) {
    assert (extra >= 0);
    if (buf->len + extra >= buf->size) {
        buffer_resize(buf, buf->len + extra);
    }
}
static inline void buffer_setlen(buffer_t *buf, ssize_t len) {
    assert (buf->size > len);
    buf->len = len;
    buf->data[len] = '\0';
}
static inline void buffer_extend(buffer_t *buf, ssize_t extra) {
    buffer_ensure(buf, extra);
    buf->len += extra;
    buf->data[buf->len] = '\0';
}
static inline void buffer_extendch(buffer_t *buf, ssize_t extra, int c) {
    buffer_ensure(buf, extra);
    memset(buf->data + buf->len, c, extra);
    buf->len += extra;
    buf->data[buf->len] = '\0';
}


static inline void buffer_add(buffer_t *buf, const void *data, ssize_t len) {
    buffer_ensure(buf, len);
    memcpy(buf->data + buf->len, data, len);
    buf->len += len;
    buf->data[buf->len] = '\0';
}
static inline void buffer_addstr(buffer_t *buf, const char *s) {
    buffer_add(buf, s, m_strlen(s));
}
static inline void buffer_addbuf(buffer_t *buf, buffer_t *buf2) {
    buffer_add(buf, buf2->data, buf2->len);
}
static inline void buffer_addch(buffer_t *buf, int c) {
    buffer_extendch(buf, 1, c);
}

static inline void buffer_reset(buffer_t *buf) {
    if (buf->len) {
        buf->len = 0;
        buf->data[0] = '\0';
    }
}

void buffer_splice(buffer_t *, ssize_t pos, ssize_t len, const void *, ssize_t);
void buffer_consume_upto(buffer_t *, const char *s);

ssize_t buffer_addvf(buffer_t *buf, const char *fmt, va_list)
    __attribute__((format(printf, 2, 0)));

ssize_t buffer_addf(buffer_t *buf, const char *fmt, ...)
    __attribute__((format(printf, 2, 3)));


/****** LEGACY BUFFERS *******/

typedef struct {
    char *data;          /* pointer to data */
    char *dptr;          /* current read/write position */
    ssize_t dsize;       /* length of data */
    int destroy;         /* destroy `data' when done? */
} BUFFER;

BUFFER *mutt_buffer_init(BUFFER *);
void mutt_buffer_free(BUFFER **);

BUFFER *mutt_buffer_from(BUFFER *, const char *);
void mutt_buffer_add(BUFFER *, const char *, ssize_t);
static inline void mutt_buffer_addstr(BUFFER *b, const char *s) {
    mutt_buffer_add(b, s, m_strlen(s));
}

static inline void mutt_buffer_addch(BUFFER *b, char c) {
    mutt_buffer_add(b, &c, 1);
}

static inline void mutt_buffer_reset(BUFFER *b) {
    *(b->dptr = b->data) = '\0';
}

#endif /* MUTT_LIB_LIB_BUFFER_H */
